# -*- coding: utf-8 -*-
"""
Fitness function
Author: Rafal Biedrzycki
"""

import numpy as np

from simulator import initialize, simpyt

PEN_MULT = 1e9

TIME_STEP=15
#TIME_STEP=1

params_f_name='simulator/params_60.txt'
if TIME_STEP == 15:
    weather_f_name = 'simulator/input-data/weather_d4_m3_24h_quarter.csv'
    price_f_name = 'simulator/input-data/price_4_people_24h_quarter.csv'    
else:
    weather_f_name = 'simulator/input-data/weather_d4_m3_24h.csv'
    price_f_name = 'simulator/input-data/price_4_people_24h.csv'    


(param, T_a1, cost) = initialize.load_data(weather_f_name, price_f_name, params_f_name)
a =  param.loc['a'][1] 
EER_div_step = 2*(1 + a*0.03)*1000*(60/TIME_STEP)


LOWER=1
UPPER=3

def fitness_fun_bin_repr( on_seq, lower, upper ):
    result = simpyt.simulate( param, T_a1, on_seq )

    al1     = on_seq * cost 
    T_hl2   = 2*result[:,3] - result[:,0]
    Q_mod   = 0.0611*3719.2*(T_hl2 - result[:,0])  
    price =sum(al1 * Q_mod)/EER_div_step 
    Ts = result[:, 4] 
    penalty = ((Ts[Ts>upper]-upper)**2).sum() + ((Ts[Ts<lower]-lower)**2).sum()
    penalty *= PEN_MULT
    fitness = price + penalty
    return fitness

def fitness_fun_bin_repr_wrapper(on_seq):
    return fitness_fun_bin_repr(on_seq, LOWER, UPPER)

    
NUM_OF_H=24

GEN_LEN = int(NUM_OF_H*60/TIME_STEP)

on_seq = np.repeat(False, GEN_LEN) #exemplary sequence- compressor is always off


fitness = fitness_fun_bin_repr_wrapper(on_seq)


def fitness_fun_cont_repr( temp_ref_points, lower, upper ):
    resultTuple = simpyt.simulate_temp_ref_points( param, T_a1, temp_ref_points )
    result=resultTuple[0]
    on_seq = resultTuple[1]
    al1     = on_seq * cost 
    T_hl2   = 2*result[:,3] - result[:,0]
    Q_mod   = 0.0611*3719.2*(T_hl2 - result[:,0])  
    price =sum(al1 * Q_mod)/EER_div_step 
    Ts = result[:, 4] 
    penalty = ((Ts[Ts>upper]-upper)**2).sum() + ((Ts[Ts<lower]-lower)**2).sum()
    penalty *= PEN_MULT
    fitness = price + penalty
    return fitness

def fitness_fun_cont_repr_wrapper(temp_ref_points):
    return fitness_fun_cont_repr(temp_ref_points, LOWER, UPPER)

temp_ref_points=np.repeat(2, NUM_OF_H) #exemplary sequence- assume we request always 2 degrees C
fitness=fitness_fun_cont_repr_wrapper(temp_ref_points)
