"""Author: Kamil Wołos, Michał Suchoński, Kamil Kwiatkowski (Euros Energy)
   modifications: Rafał Biedrzycki
   """

import numpy as np
from scipy.integrate import odeint 

def T_ev2(x, alpha, temp):
    if (alpha == 0):
        t = 2 * x[3] - x[0] - 0.5
    else:
        t = temp
    return t


def simulate(param, T_a1, alpha):
    rho_p = param.loc['rho_p'].iloc[0]
    temp =  param.loc['a'].iloc[0]         
    N_p =  param.loc['N_p'].iloc[0]        
    
    D_p   = param.loc['D_p'].iloc[0]     
    S_p   = param.loc['S_p'].iloc[0]     
    W_p   = param.loc['W_p'].iloc[0]     
    
    
    D_S = param.loc['D_S'].iloc[0]
    S_S = param.loc['S_S'].iloc[0]
    W_S = param.loc['W_S'].iloc[0]  

    
    A_S1 = 2*D_S*W_S + 2*S_S*W_S + 2*D_S*S_S  
    A_p  = (2*D_p*W_p + 2*S_p*W_p + 2*D_p*S_p)*N_p      
    A_h  = param.loc['A_h'].iloc[0]                     
    A_e  = param.loc['A_e'].iloc[0]                          

    
    l_z     = param.loc['l_z'].iloc[0]                   
    l_y     = param.loc['l_y'].iloc[0]                      
    delta_W = param.loc['delta_W'].iloc[0]                   
    n_W     = param.loc['n_W'].iloc[0]                     
    n_H     = 0.5*n_W + 1                             
    delta_F = param.loc['delta_F'].iloc[0]                 
    delta_H = delta_F                                  
    d_h     = (4*delta_H*l_y)/(2*delta_H + l_y)        
    c_L     = param.loc['c_L'].iloc[0]                     
    c_S     = param.loc['c_S'].iloc[0]                      
    Deltah_SL = param.loc['Deltah_SL'].iloc[0]               
    T_SL    = param.loc['T_SL'].iloc[0]                     
    lambda_F= param.loc['lambda_F'].iloc[0]                 
    A_W     = l_z*l_y                                   
    V_W     = l_z*l_y*delta_W                          
    V_H     = delta_H*l_y*l_z                          
    V_F     = delta_F*l_z*l_y                          
    
    
    lambda_H = param.loc['lambda_H'].iloc[0] 
    Nu_H     = param.loc['Nu_H'].iloc[0]
    alpha_H  = (Nu_H*lambda_H)/d_h

    rho_W    = param.loc['rho_W'].iloc[0]
    c_W      = param.loc['c_W'].iloc[0]       
    lambda_W = param.loc['lambda_W'].iloc[0]

    rho_F = param.loc['rho_F'].iloc[0]     

    DeltaT_SL = param.loc['DeltaT_SL'].iloc[0]
    B = param.loc['B'].iloc[0]                                               
    c_mp = lambda T: (c_L + c_S)/2 + ((c_L - c_S)/2)*np.tanh(T - T_SL)  
    c_tp = lambda T: (Deltah_SL/2)*(np.tanh(B*(T - T_SL + DeltaT_SL/2))-np.tanh(B*(T-T_SL-DeltaT_SL/2)))  
    c_F = lambda T: c_mp(T) + c_tp(T)                   

    k_FW = 1.0/(delta_W/(2*lambda_W) + delta_W/lambda_F)
    k_HW = 1.0/(delta_W/(2*lambda_W) + 1.0/alpha_H)     
    k_S1 = param.loc['k_S1'].iloc[0]                         
    k_p  = param.loc['k_p'].iloc[0]                         
    k_h  = param.loc['k_h'].iloc[0]                          
    k_e  = param.loc['k_e'].iloc[0]                          

    c_1  = param.loc['c_1'].iloc[0]                          
    c_pg = param.loc['c_pg'].iloc[0]                         
    c_p  = param.loc['c_p'].iloc[0]                          

    rho_H  = param.loc['rho_H'].iloc[0]                      
    rho_sg = param.loc['rho_sg'].iloc[0]                     

    M_H  = rho_H*V_H                                    
    M_F  = rho_F*V_F                                    
    M_W  = rho_W*V_W                                    
    M_hl = param.loc['M_hl'].iloc[0]                     
    M_sg = rho_sg*(D_S*S_S*W_S - N_p*D_p*S_p*W_p)       
    M_p  = rho_p*D_p*S_p*W_p*N_p                        

    m_g = 1100.0/3600.0*rho_sg                          
    m_l = 220.0/3600.0                                  
    m_H = m_l/(2*n_H)                                   
    
    beta_H   = m_H /M_H  
    gamma_H  = k_HW * A_W/ (M_H * c_1) 
    gamma_F  = lambda T: k_FW * A_W / (M_F * c_F(T))  
    gamma_FW = k_FW * A_W/ (M_W * c_W)  
    gamma_HW = k_HW * A_W/ (M_W * c_W) 
    beta_hl  =  m_l/ M_hl 
    gamma_hl = k_h * A_h/ (M_hl * c_1)  
    beta_s   = m_g / M_sg  
    gamma_sp = k_p * A_p / (M_sg * c_pg) 
    gamma_s1 = k_S1 * A_S1/ (M_sg * c_pg)               
    gamma_p  = k_p * A_p/ (M_p * c_p)  
    gamma_r  = 1.0/ c_p 
    eps_hl   = 1.0/(1.0+(2*m_g*c_pg)/(k_h*A_h)) 
    eps_e    = 1.0/(0.5+(m_l*c_1)/(k_e*A_e)) 

    q_r0 = param.loc['q_r0'].iloc[0] 
    b    = param.loc['b'].iloc[0] 

    T_H   = param.loc['T_H'].iloc[0]       
    T_F   = param.loc['T_F'].iloc[0]       
    T_W   = param.loc['T_W'].iloc[0]       
    T_hl  = param.loc['T_hl'].iloc[0]     
    T_s   = param.loc['T_s'].iloc[0]       
    T_p   = param.loc['T_p'].iloc[0]       
  
      
    def model(x, t, Ta, alpha):
        f_H  = beta_H * (eps_e * T_ev2(x, alpha, temp) + (1 - eps_e)*(2 * x[3] - x[0]) - x[0]) + gamma_H * (x[2]-x[0])  
        f_F  = gamma_F(x[1]) * (x[2] - x[1]) 
        f_W  = gamma_FW * (x[1] - x[2]) + gamma_HW * (x[0] - x[2])  
        f_hl = 2 * beta_hl * (x[0] - x[3]) - gamma_hl * (1 - eps_hl) * (x[3] - x[4])  
        f_s  = - 2 * beta_s * eps_hl * (x[4] - x[3]) + gamma_sp * (x[5] - x[4]) + gamma_s1 * (Ta - x[4])  
        f_p  = gamma_p * (x[4] - x[5]) + gamma_r * q_r0 * np.exp(b * (x[5] - 273.15)) 
        return np.array([f_H, f_F, f_W, f_hl, f_s, f_p])
        
    x_poprz0 = np.array([T_H, T_F, T_W, T_hl, T_s, T_p])
    t=T_a1['czas']*60  
    tspan    = [t[0], t[1]]

    temp_outd=T_a1['temp']
    z = np.zeros((t.size, len(x_poprz0)))
    
    n = t.size
    
    for i in range(0, n):
        zzz      = odeint(model, x_poprz0, tspan, args =(temp_outd[i], alpha[i]), rtol=0.02, atol=0.02)
        z[i, :]  = zzz[1]
        x_poprz0 = zzz[1]
    return z


def simulate_temp_ref_points(param, T_a1, tset):
    rho_p = param.loc['rho_p'].iloc[0]
    temp =  param.loc['a'].iloc[0]         
    N_p =  param.loc['N_p'].iloc[0]        
    
    D_p   = param.loc['D_p'].iloc[0]     
    S_p   = param.loc['S_p'].iloc[0]     
    W_p   = param.loc['W_p'].iloc[0]     
    
    
    D_S = param.loc['D_S'].iloc[0]
    S_S = param.loc['S_S'].iloc[0]
    W_S = param.loc['W_S'].iloc[0]  

    
    A_S1 = 2*D_S*W_S + 2*S_S*W_S + 2*D_S*S_S  
    A_p  = (2*D_p*W_p + 2*S_p*W_p + 2*D_p*S_p)*N_p      
    A_h  = param.loc['A_h'].iloc[0]                     
    A_e  = param.loc['A_e'].iloc[0]                          

    
    l_z     = param.loc['l_z'].iloc[0]                   
    l_y     = param.loc['l_y'].iloc[0]                      
    delta_W = param.loc['delta_W'].iloc[0]                   
    n_W     = param.loc['n_W'].iloc[0]                     
    n_H     = 0.5*n_W + 1                             
    delta_F = param.loc['delta_F'].iloc[0]                 
    delta_H = delta_F                                  
    d_h     = (4*delta_H*l_y)/(2*delta_H + l_y)        
    c_L     = param.loc['c_L'].iloc[0]                     
    c_S     = param.loc['c_S'].iloc[0]                      
    Deltah_SL = param.loc['Deltah_SL'].iloc[0]               
    T_SL    = param.loc['T_SL'].iloc[0]                     
    lambda_F= param.loc['lambda_F'].iloc[0]                 
    A_W     = l_z*l_y                                   
    V_W     = l_z*l_y*delta_W                          
    V_H     = delta_H*l_y*l_z                          
    V_F     = delta_F*l_z*l_y                          
    
    
    lambda_H = param.loc['lambda_H'].iloc[0] 
    Nu_H     = param.loc['Nu_H'].iloc[0]
    alpha_H  = (Nu_H*lambda_H)/d_h

    rho_W    = param.loc['rho_W'].iloc[0]
    c_W      = param.loc['c_W'].iloc[0]       
    lambda_W = param.loc['lambda_W'].iloc[0]

    rho_F = param.loc['rho_F'].iloc[0]     

    DeltaT_SL = param.loc['DeltaT_SL'].iloc[0]
    B = param.loc['B'].iloc[0]                                               
    c_mp = lambda T: (c_L + c_S)/2 + ((c_L - c_S)/2)*np.tanh(T - T_SL)  
    c_tp = lambda T: (Deltah_SL/2)*(np.tanh(B*(T - T_SL + DeltaT_SL/2))-np.tanh(B*(T-T_SL-DeltaT_SL/2)))  
    c_F = lambda T: c_mp(T) + c_tp(T)                   

    k_FW = 1.0/(delta_W/(2*lambda_W) + delta_W/lambda_F)
    k_HW = 1.0/(delta_W/(2*lambda_W) + 1.0/alpha_H)     
    k_S1 = param.loc['k_S1'].iloc[0]                         
    k_p  = param.loc['k_p'].iloc[0]                         
    k_h  = param.loc['k_h'].iloc[0]                          
    k_e  = param.loc['k_e'].iloc[0]                          

    c_1  = param.loc['c_1'].iloc[0]                          
    c_pg = param.loc['c_pg'].iloc[0]                         
    c_p  = param.loc['c_p'].iloc[0]                          

    rho_H  = param.loc['rho_H'].iloc[0]                      
    rho_sg = param.loc['rho_sg'].iloc[0]                     

    M_H  = rho_H*V_H                                    
    M_F  = rho_F*V_F                                    
    M_W  = rho_W*V_W                                    
    M_hl = param.loc['M_hl'].iloc[0]                     
    M_sg = rho_sg*(D_S*S_S*W_S - N_p*D_p*S_p*W_p)       
    M_p  = rho_p*D_p*S_p*W_p*N_p                        

    m_g = 1100.0/3600.0*rho_sg                          
    m_l = 220.0/3600.0                                  
    m_H = m_l/(2*n_H)                                   
    
    beta_H   = m_H /M_H  
    gamma_H  = k_HW * A_W/ (M_H * c_1) 
    gamma_F  = lambda T: k_FW * A_W / (M_F * c_F(T))  
    gamma_FW = k_FW * A_W/ (M_W * c_W)  
    gamma_HW = k_HW * A_W/ (M_W * c_W) 
    beta_hl  =  m_l/ M_hl 
    gamma_hl = k_h * A_h/ (M_hl * c_1)  
    beta_s   = m_g / M_sg  
    gamma_sp = k_p * A_p / (M_sg * c_pg) 
    gamma_s1 = k_S1 * A_S1/ (M_sg * c_pg)               
    gamma_p  = k_p * A_p/ (M_p * c_p)  
    gamma_r  = 1.0/ c_p 
    eps_hl   = 1.0/(1.0+(2*m_g*c_pg)/(k_h*A_h)) 
    eps_e    = 1.0/(0.5+(m_l*c_1)/(k_e*A_e)) 

    q_r0 = param.loc['q_r0'].iloc[0] 
    b    = param.loc['b'].iloc[0] 

    T_H   = param.loc['T_H'].iloc[0]       
    T_F   = param.loc['T_F'].iloc[0]       
    T_W   = param.loc['T_W'].iloc[0]       
    T_hl  = param.loc['T_hl'].iloc[0]     
    T_s   = param.loc['T_s'].iloc[0]       
    T_p   = param.loc['T_p'].iloc[0]       
  
      
    def model(x, t, Ta, alpha):
        f_H  = beta_H * (eps_e * T_ev2(x, alpha, temp) + (1 - eps_e)*(2 * x[3] - x[0]) - x[0]) + gamma_H * (x[2]-x[0])  
        f_F  = gamma_F(x[1]) * (x[2] - x[1]) 
        f_W  = gamma_FW * (x[1] - x[2]) + gamma_HW * (x[0] - x[2])  
        f_hl = 2 * beta_hl * (x[0] - x[3]) - gamma_hl * (1 - eps_hl) * (x[3] - x[4])  
        f_s  = - 2 * beta_s * eps_hl * (x[4] - x[3]) + gamma_sp * (x[5] - x[4]) + gamma_s1 * (Ta - x[4])  
        f_p  = gamma_p * (x[4] - x[5]) + gamma_r * q_r0 * np.exp(b * (x[5] - 273.15)) 
        return np.array([f_H, f_F, f_W, f_hl, f_s, f_p])
        
    x_poprz0 = np.array([T_H, T_F, T_W, T_hl, T_s, T_p])
    t=T_a1['czas']*60  
    tspan    = [t[0], t[1]]

    temp_outd=T_a1['temp']
    z = np.zeros((t.size, len(x_poprz0)))
    
    n = t.size
    on_seq = np.repeat(False, t.size)
    cur_hour = 0
    timesteps_in_h = int(t.size/len(tset))

    
    for i in range(0, n):
        cur_tset = tset[cur_hour]
        if x_poprz0[4] >= cur_tset:
            on_seq[i]=True

        zzz      = odeint(model, x_poprz0, tspan, args =(temp_outd[i], on_seq[i]), rtol=0.02, atol=0.02)
        z[i, :]  = zzz[1]
        x_poprz0 = zzz[1]
        if (i+1) % timesteps_in_h == 0:
            cur_hour += 1
        
    return (z, on_seq)
